@@ -58,18 +58,22 @@ showed_message_cb (GtrTab * tab, GtrMsg * msg, GtrMessageTable * table)
 {
   GtkTreePath *path;
   GtkTreeSelection *selection;
-  GtkTreeIter iter;
+  GtkTreeIter iter, child_iter;
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (table->priv->treeview));
+  gtr_message_table_get_message_iter (table->priv->store, msg, &child_iter);
 
-  path = gtk_tree_row_reference_get_path (_gtr_msg_get_row_reference (msg));
-
-  gtk_tree_model_get_iter (table->priv->sort_model, &iter, path);
+  gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT
+                                                  (table->
+                                                   priv->sort_model),
+                                                  &iter, &child_iter);
 
   gtk_tree_selection_select_iter (selection, &iter);
+  path = gtk_tree_model_get_path (table->priv->sort_model, &iter);
 
   gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (table->priv->treeview),
                                 path, NULL, TRUE, 0.5, 0.0);
+
   gtk_tree_path_free (path);
 }
 
@@ -79,7 +83,7 @@ gtr_message_table_selection_changed (GtkTreeSelection *selection,
 {
   GtkTreeIter iter;
   GtkTreeModel *model;
-  GList *msg = NULL;
+  GtrMsg *msg;
   GList *current_msg = NULL;
   GtrPo *po;
 
@@ -94,7 +98,7 @@ gtr_message_table_selection_changed (GtkTreeSelection *selection,
                           GTR_MESSAGE_TABLE_MODEL_POINTER_COLUMN, &msg, -1);
 
       if (msg != NULL
-          && g_utf8_collate (gtr_msg_get_msgid (msg->data),
+          && g_utf8_collate (gtr_msg_get_msgid (msg),
                              gtr_msg_get_msgid (current_msg->data)))
         {
           g_signal_handlers_block_by_func (table->priv->tab, showed_message_cb, table);
@@ -108,18 +112,17 @@ gtr_message_table_selection_changed (GtkTreeSelection *selection,
 static void
 message_changed_cb (GtrTab * tab, GtrMsg * msg, GtrMessageTable * table)
 {
-  GtkTreePath *sort_path, *path;
-  GtkTreeRowReference *row;
+  GtkTreePath *path;
+  GtkTreeIter iter;
 
-  row = _gtr_msg_get_row_reference (msg);
-  sort_path = gtk_tree_row_reference_get_path (row);
+  if (!gtr_message_table_get_message_iter (GTR_MESSAGE_TABLE_MODEL (table->priv->store),
+                                           msg, &iter))
+    return;
 
-  path =
-    gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT
-                                                    (table->priv->sort_model),
-                                                    sort_path);
-
-  gtr_message_table_model_update_row (table->priv->store, path);
+  path = gtk_tree_model_get_path (GTK_TREE_MODEL (table->priv->store), &iter);
+  gtr_message_table_model_update_row (GTR_MESSAGE_TABLE_MODEL (table->priv->store),
+                                      path);
+  gtk_tree_path_free (path);
 }
 
 static gint
@@ -149,116 +152,23 @@ model_compare_by_status (GtkTreeModel * model,
   return compare_by_status (a_status, b_status, a_pos, b_pos);
 }
 
-static gint
-list_compare_by_status (gconstpointer a, gconstpointer b)
-{
-  gint a_status, b_status, a_pos, b_pos;
-
-  a_status = gtr_msg_get_status (GTR_MSG (a));
-  b_status = gtr_msg_get_status (GTR_MSG (b));
-
-  a_pos = gtr_msg_get_po_position (GTR_MSG (a));
-  b_pos = gtr_msg_get_po_position (GTR_MSG (b));
-
-  return compare_by_status (a_status, b_status, a_pos, b_pos);
-}
-
-static gint
-list_compare_by_position (gconstpointer a, gconstpointer b)
-{
-  gint a_pos, b_pos;
-  a_pos = gtr_msg_get_po_position (GTR_MSG (a));
-  b_pos = gtr_msg_get_po_position (GTR_MSG (b));
-
-  return a_pos - b_pos;
-}
-
-static gint
-list_compare_by_original (gconstpointer a, gconstpointer b)
-{
-  const gchar *a_original, *b_original;
-
-  a_original = gtr_msg_get_msgid (GTR_MSG (a));
-  b_original = gtr_msg_get_msgid (GTR_MSG (b));
-
-  return g_utf8_collate (a_original, b_original);
-}
-
-static gint
-list_compare_by_translation (gconstpointer a, gconstpointer b)
-{
-  const gchar *a_translated, *b_translated;
-
-  a_translated = gtr_msg_get_msgstr (GTR_MSG (a));
-  b_translated = gtr_msg_get_msgstr (GTR_MSG (b));
-
-  return g_utf8_collate (a_translated, b_translated);
-}
-
 static void
-sort_message_list (GtkTreeViewColumn * column, GtrMessageTable * table)
+gtr_message_table_init (GtrMessageTable * table)
 {
-  GtrPo *po;
-  GList *messages;
-  gint sort_column;
-
-  po = gtr_tab_get_po (table->priv->tab);
-  messages = gtr_po_get_messages (po);
-
-  sort_column = gtk_tree_view_column_get_sort_column_id (column);
-  switch (sort_column)
-    {
-    case GTR_MESSAGE_TABLE_MODEL_ID_COLUMN:
-      messages = g_list_sort (messages, list_compare_by_position);
-      break;
-    case GTR_MESSAGE_TABLE_MODEL_STATUS_COLUMN:
-      messages = g_list_sort (messages, list_compare_by_status);
-      break;
-    case GTR_MESSAGE_TABLE_MODEL_ORIGINAL_COLUMN:
-      messages = g_list_sort (messages, list_compare_by_original);
-      break;
-    case GTR_MESSAGE_TABLE_MODEL_TRANSLATION_COLUMN:
-      messages = g_list_sort (messages, list_compare_by_translation);
-      break;
-    }
-
-  if (gtk_tree_view_column_get_sort_order (column) == GTK_SORT_DESCENDING)
-    messages = g_list_reverse (messages);
-
-  gtr_po_set_messages (po, messages);
-
-  gtr_tab_message_go_to (table->priv->tab,
-                         g_list_first (messages), FALSE, GTR_TAB_MOVE_NONE);
-}
-
-static void
-gtr_message_table_draw (GtrMessageTable * table)
-{
-  GtrMessageTablePrivate *priv = table->priv;
-
+  GtkWidget *scrolledwindow;
   GtkTreeViewColumn *column;
   GtkCellRenderer *renderer;
   GtkTreeSelection *selection;
 
-  priv->store = gtr_message_table_model_new ();
+  table->priv = GTR_MESSAGE_TABLE_GET_PRIVATE (table);
 
-  priv->sort_model =
-    gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (priv->store));
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (table),
+                                  GTK_ORIENTATION_VERTICAL);
+
+  GtrMessageTablePrivate *priv = table->priv;
 
   priv->treeview = gtk_tree_view_new ();
 
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (priv->sort_model),
-                                        GTR_MESSAGE_TABLE_MODEL_ID_COLUMN,
-                                        GTK_SORT_ASCENDING);
-
-  gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE
-                                           (priv->sort_model), NULL, NULL,
-                                           NULL);
-
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (priv->sort_model),
-                                   GTR_MESSAGE_TABLE_MODEL_STATUS_COLUMN,
-                                   model_compare_by_status, NULL, NULL);
-
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (priv->treeview), TRUE);
 
   renderer = gtk_cell_renderer_pixbuf_new ();
@@ -273,10 +183,6 @@ gtr_message_table_draw (GtrMessageTable * table)
   gtk_tree_view_column_set_resizable (column, FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (priv->treeview), column);
 
-  /* Resort underlying GList when column header clicked */
-  g_signal_connect (G_OBJECT (column), "clicked",
-                    G_CALLBACK (sort_message_list), table);
-
   renderer = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new_with_attributes (_("ID"),
                                                      renderer,
@@ -289,10 +195,6 @@ gtr_message_table_draw (GtrMessageTable * table)
   gtk_tree_view_column_set_resizable (column, FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (priv->treeview), column);
 
-  /* Resort underlying GList when column header clicked */
-  g_signal_connect (G_OBJECT (column), "clicked",
-                    G_CALLBACK (sort_message_list), table);
-
   renderer = gtk_cell_renderer_text_new ();
   g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
   if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL)
@@ -309,10 +211,6 @@ gtr_message_table_draw (GtrMessageTable * table)
   gtk_tree_view_column_set_resizable (column, TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (priv->treeview), column);
 
-  /* Resort underlying GList when column header clicked */
-  g_signal_connect (G_OBJECT (column), "clicked",
-                    G_CALLBACK (sort_message_list), table);
-
   renderer = gtk_cell_renderer_text_new ();
   g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 
@@ -328,33 +226,13 @@ gtr_message_table_draw (GtrMessageTable * table)
   gtk_tree_view_column_set_resizable (column, TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (priv->treeview), column);
 
-  /* Resort underlying GList when column header clicked */
-  g_signal_connect (G_OBJECT (column), "clicked",
-                    G_CALLBACK (sort_message_list), table);
-
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
   gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 
   g_signal_connect (G_OBJECT (selection), "changed",
                     G_CALLBACK (gtr_message_table_selection_changed), table);
-}
-
-
-static void
-gtr_message_table_init (GtrMessageTable * table)
-{
-  GtkWidget *scrolledwindow;
-
-  table->priv = GTR_MESSAGE_TABLE_GET_PRIVATE (table);
-
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (table),
-                                  GTK_ORIENTATION_VERTICAL);
-
-  gtr_message_table_draw (table);
 
   scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
-                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_box_pack_start (GTK_BOX (table), scrolledwindow, TRUE, TRUE, 0);
   gtk_widget_show (scrolledwindow);
 
@@ -411,35 +289,124 @@ gtr_message_table_new (GtkWidget * tab)
  * sort them.
  */
 void
-gtr_message_table_populate (GtrMessageTable * table, GList * messages)
+gtr_message_table_populate (GtrMessageTable * table, GtrMessageContainer * container)
 {
-  GtkTreeIter iter, sort_iter;
-  GtkTreePath *path;
-  GtkTreeRowReference *row;
-
   g_return_if_fail (table != NULL);
-  g_return_if_fail (messages != NULL);
+  g_return_if_fail (container != NULL);
 
-  while (messages)
+  if (table->priv->store)
     {
-      gtr_message_table_model_append (table->priv->store,
-                                      messages->data, &iter);
-
-      gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT
-                                                      (table->
-                                                       priv->sort_model),
-                                                      &sort_iter, &iter);
-      path = gtk_tree_model_get_path (table->priv->sort_model, &sort_iter);
-      row = gtk_tree_row_reference_new (table->priv->sort_model, path);
-      gtk_tree_path_free (path);
-
-      _gtr_msg_set_row_reference (GTR_MSG (messages->data), row);
-
-      messages = g_list_next (messages);
+      gtk_tree_view_set_model (GTK_TREE_VIEW (table->priv->treeview), NULL);
+      g_object_unref (table->priv->sort_model);
+      g_object_unref (table->priv->store);
     }
-  /*
-   * It is much faster set the model after list population
-   */
+
+  table->priv->store = gtr_message_table_model_new (container);
+  table->priv->sort_model =
+    gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (table->priv->store));
+
+  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE
+                                        (table->priv->sort_model),
+                                        GTR_MESSAGE_TABLE_MODEL_ID_COLUMN,
+                                        GTK_SORT_ASCENDING);
+
+  gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE
+                                           (table->priv->sort_model),
+                                           NULL, NULL, NULL);
+
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (table->priv->sort_model),
+                                   GTR_MESSAGE_TABLE_MODEL_STATUS_COLUMN,
+                                   model_compare_by_status, NULL, NULL);
+
   gtk_tree_view_set_model (GTK_TREE_VIEW (table->priv->treeview),
                            table->priv->sort_model);
 }
+
+GtrMsg *
+gtr_message_table_navigate (GtrMessageTable * table,
+                            GtrMessageTableNavigation navigation,
+                            GtrMessageTableNavigationFunc func)
+{
+  GtkTreeSelection *selection;
+  GtkTreeModel *model;
+  GtkTreePath *path;
+  GtkTreeIter iter;
+  GtrMsg *msg;
+  gboolean cont = TRUE;
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (table->priv->treeview));
+
+  if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+    return NULL;
+
+  switch (navigation)
+    {
+    case GTR_NAVIGATE_FIRST:
+      if (!gtk_tree_model_get_iter_first (model, &iter))
+        return NULL;
+      break;
+    case GTR_NAVIGATE_LAST:
+      {
+        gint n_children;
+
+        n_children = gtk_tree_model_iter_n_children (model, NULL);
+
+        if (n_children <= 0)
+          return NULL;
+
+        if (!gtk_tree_model_iter_nth_child (model, &iter, NULL, n_children - 1))
+          return NULL;
+      }
+      break;
+    case GTR_NAVIGATE_NEXT:
+      if (func)
+        {
+          while (cont)
+            {
+              if (!gtk_tree_model_iter_next (model, &iter))
+                return NULL;
+
+              gtk_tree_model_get (model, &iter,
+                                  GTR_MESSAGE_TABLE_MODEL_POINTER_COLUMN, &msg,
+                                  -1);
+
+              if (func (msg))
+                cont = FALSE;
+            }
+        }
+      else if (!gtk_tree_model_iter_next (model, &iter))
+        return NULL;
+
+      break;
+    case GTR_NAVIGATE_PREV:
+      if (func)
+        {
+          while (cont)
+            {
+              if (!gtk_tree_model_iter_previous (model, &iter))
+                return NULL;
+
+              gtk_tree_model_get (model, &iter,
+                                  GTR_MESSAGE_TABLE_MODEL_POINTER_COLUMN, &msg,
+                                  -1);
+
+              if (func (msg))
+                cont = FALSE;
+            }
+        }
+      else if (!gtk_tree_model_iter_previous (model, &iter))
+        return NULL;
+      break;
+    }
+
+  gtk_tree_selection_select_iter (selection, &iter);
+  path = gtk_tree_model_get_path (model, &iter);
+  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (table->priv->treeview),
+                                path, NULL, TRUE, 0.5, 0.0);
+
+  gtk_tree_model_get (model, &iter,
+                      GTR_MESSAGE_TABLE_MODEL_POINTER_COLUMN, &msg,
+                      -1);
+
+  return msg;
+}
