@@ -125,6 +125,7 @@ enum
   SHOWED_MESSAGE,
   MESSAGE_CHANGED,
   MESSAGE_EDITION_FINISHED,
+  SELECTION_CHANGED,
   LAST_SIGNAL
 };
 
@@ -431,9 +432,6 @@ gtr_tab_append_msgstr_page (const gchar * tab_label,
 
   gtk_container_add (GTK_CONTAINER (scroll), widget);
 
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
-                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll),
                                        GTK_SHADOW_IN);
 
@@ -545,6 +543,12 @@ emit_message_changed_signal (GtkTextBuffer * buf, GtrTab * tab)
   g_signal_emit (G_OBJECT (tab), signals[MESSAGE_CHANGED], 0, msg->data);
 }
 
+static void
+emit_selection_changed (GtkTextBuffer * buf, GParamSpec * spec, GtrTab * tab)
+{
+  g_signal_emit (G_OBJECT (tab), signals[SELECTION_CHANGED], 0);
+}
+
 static void
 update_status (GtrTab * tab, GtrMsg * msg, gpointer useless)
 {
@@ -630,6 +634,8 @@ gtr_tab_add_msgstr_tabs (GtrTab * tab)
 
       g_signal_connect_after (buf, "end_user_action",
                               G_CALLBACK (emit_message_changed_signal), tab);
+      g_signal_connect (buf, "notify::has-selection",
+                        G_CALLBACK (emit_selection_changed), tab);
       i++;
       g_free (label);
     }
@@ -672,14 +678,45 @@ on_layout_changed (GdlDockMaster *master,
 }
 
 static void
-gtr_tab_draw (GtrTab *tab)
+extension_added (PeasExtensionSet *extensions,
+                 PeasPluginInfo   *info,
+                 PeasExtension    *exten,
+                 GtrTab           *tab)
+{
+  gtr_tab_activatable_activate (GTR_TAB_ACTIVATABLE (exten));
+}
+
+static void
+extension_removed (PeasExtensionSet *extensions,
+                   PeasPluginInfo   *info,
+                   PeasExtension    *exten,
+                   GtrTab           *tab)
+{
+  gtr_tab_activatable_deactivate (GTR_TAB_ACTIVATABLE (exten));
+}
+
+static void
+gtr_tab_init (GtrTab * tab)
 {
   GtkWidget *hbox;
   GtkWidget *vertical_box;
   GtkWidget *msgid_label;
   GtkWidget *scroll;
   GtkWidget *dockbar;
-  GtrTabPrivate *priv = tab->priv;
+  GtrTabPrivate *priv;
+
+  tab->priv = GTR_TAB_GET_PRIVATE (tab);
+  priv = tab->priv;
+
+  priv->ui_settings = g_settings_new ("org.gnome.gtranslator.preferences.ui");
+  priv->files_settings = g_settings_new ("org.gnome.gtranslator.preferences.files");
+  priv->editor_settings = g_settings_new ("org.gnome.gtranslator.preferences.editor");
+  priv->state_settings = g_settings_new ("org.gnome.gtranslator.state.window");
+
+  g_signal_connect (tab, "message-changed", G_CALLBACK (update_status), NULL);
+
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (tab),
+                                  GTK_ORIENTATION_VERTICAL);
 
   /* Docker */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
@@ -742,8 +779,6 @@ gtr_tab_draw (GtrTab *tab)
 
   gtk_container_add (GTK_CONTAINER (scroll), priv->text_msgid);
 
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
-                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll),
                                        GTK_SHADOW_IN);
 
@@ -760,9 +795,6 @@ gtr_tab_draw (GtrTab *tab)
   gtk_container_add (GTK_CONTAINER (priv->text_plural_scroll),
                      priv->text_msgid_plural);
 
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW
-                                  (priv->text_plural_scroll),
-                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW
                                        (priv->text_plural_scroll),
                                        GTK_SHADOW_IN);
@@ -810,64 +842,28 @@ gtr_tab_draw (GtrTab *tab)
                       NULL,
                       GTR_TAB_PLACEMENT_RIGHT,
                       FALSE);
-}
-
-static void
-extension_added (PeasExtensionSet *extensions,
-                 PeasPluginInfo   *info,
-                 PeasExtension    *exten,
-                 GtrTab           *tab)
-{
-  gtr_tab_activatable_activate (GTR_TAB_ACTIVATABLE (exten));
-}
-
-static void
-extension_removed (PeasExtensionSet *extensions,
-                   PeasPluginInfo   *info,
-                   PeasExtension    *exten,
-                   GtrTab           *tab)
-{
-  gtr_tab_activatable_deactivate (GTR_TAB_ACTIVATABLE (exten));
-}
-
-static void
-gtr_tab_init (GtrTab * tab)
-{
-  tab->priv = GTR_TAB_GET_PRIVATE (tab);
-
-  tab->priv->ui_settings = g_settings_new ("org.gnome.gtranslator.preferences.ui");
-  tab->priv->files_settings = g_settings_new ("org.gnome.gtranslator.preferences.files");
-  tab->priv->editor_settings = g_settings_new ("org.gnome.gtranslator.preferences.editor");
-  tab->priv->state_settings = g_settings_new ("org.gnome.gtranslator.state.window");
-
-  g_signal_connect (tab, "message-changed", G_CALLBACK (update_status), NULL);
-
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (tab),
-                                  GTK_ORIENTATION_VERTICAL);
-
-  gtr_tab_draw (tab);
 
   /* Manage auto save data */
-  tab->priv->autosave = g_settings_get_boolean (tab->priv->files_settings,
-                                                GTR_SETTINGS_AUTO_SAVE);
-  tab->priv->autosave = (tab->priv->autosave != FALSE);
+  priv->autosave = g_settings_get_boolean (priv->files_settings,
+                                           GTR_SETTINGS_AUTO_SAVE);
+  priv->autosave = (priv->autosave != FALSE);
 
-  tab->priv->autosave_interval = g_settings_get_int (tab->priv->files_settings,
-                                                     GTR_SETTINGS_AUTO_SAVE_INTERVAL);
-  if (tab->priv->autosave_interval <= 0)
-    tab->priv->autosave_interval = 1;
+  priv->autosave_interval = g_settings_get_int (priv->files_settings,
+                                                GTR_SETTINGS_AUTO_SAVE_INTERVAL);
+  if (priv->autosave_interval <= 0)
+    priv->autosave_interval = 1;
 
   /* Plugins */
-  tab->priv->extensions = peas_extension_set_new (PEAS_ENGINE (gtr_plugins_engine_get_default ()),
-                                                  GTR_TYPE_TAB_ACTIVATABLE,
-                                                  "tab", tab,
-                                                  NULL);
+  priv->extensions = peas_extension_set_new (PEAS_ENGINE (gtr_plugins_engine_get_default ()),
+                                             GTR_TYPE_TAB_ACTIVATABLE,
+                                             "tab", tab,
+                                             NULL);
 
-  g_signal_connect (tab->priv->extensions,
+  g_signal_connect (priv->extensions,
                     "extension-added",
                     G_CALLBACK (extension_added),
                     tab);
-  g_signal_connect (tab->priv->extensions,
+  g_signal_connect (priv->extensions,
                     "extension-removed",
                     G_CALLBACK (extension_removed),
                     tab);
@@ -902,47 +898,13 @@ gtr_tab_dispose (GObject * object)
       priv->dispose_has_run = TRUE;
     }
 
-  if (priv->extensions != NULL)
-    {
-      g_object_unref (priv->extensions);
-      priv->extensions = NULL;
-    }
-
-  if (priv->po != NULL)
-    {
-      g_object_unref (priv->po);
-      priv->po = NULL;
-    }
-
-  if (priv->ui_settings)
-    {
-      g_object_unref (priv->ui_settings);
-      priv->ui_settings = NULL;
-    }
-
-  if (priv->files_settings != NULL)
-    {
-      g_object_unref (priv->files_settings);
-      priv->files_settings = NULL;
-    }
-
-  if (priv->editor_settings != NULL)
-    {
-      g_object_unref (priv->editor_settings);
-      priv->editor_settings = NULL;
-    }
-
-  if (priv->state_settings != NULL)
-    {
-      g_object_unref (priv->state_settings);
-      priv->state_settings = NULL;
-    }
-
-  if (priv->layout_manager)
-    {
-      g_object_unref (priv->layout_manager);
-      priv->layout_manager = NULL;
-    }
+  g_clear_object (&priv->extensions);
+  g_clear_object (&priv->po);
+  g_clear_object (&priv->ui_settings);
+  g_clear_object (&priv->files_settings);
+  g_clear_object (&priv->editor_settings);
+  g_clear_object (&priv->state_settings);
+  g_clear_object (&priv->layout_manager);
 
   G_OBJECT_CLASS (gtr_tab_parent_class)->dispose (object);
 }
@@ -1059,6 +1021,14 @@ gtr_tab_class_init (GtrTabClass * klass)
                   NULL, NULL,
                   g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1, GTR_TYPE_MSG);
+  signals[SELECTION_CHANGED] =
+    g_signal_new ("selection-changed",
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GtrTabClass, selection_changed),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 
   /* Properties */
   g_object_class_install_property (object_class,
@@ -1126,7 +1096,7 @@ gtr_tab_new (GtrPo * po)
   gtr_tab_add_msgstr_tabs (tab);
 
   gtr_message_table_populate (GTR_MESSAGE_TABLE (tab->priv->message_table),
-                              gtr_po_get_messages (tab->priv->po));
+                              GTR_MESSAGE_CONTAINER (tab->priv->po));
 
   gtk_widget_show (GTK_WIDGET (tab));
   return tab;
@@ -1235,26 +1205,12 @@ gtr_tab_get_all_views (GtrTab * tab, gboolean original, gboolean translated)
 **/
 void
 gtr_tab_message_go_to (GtrTab * tab,
-                       GList * to_go, gboolean searching, GtrTabMove move)
+                       GtrMsg * to_go, gboolean searching, GtrTabMove move)
 {
-  GtrPo *po;
-  GList *current_msg;
   static gboolean first_msg = TRUE;
 
   g_return_if_fail (tab != NULL);
-  g_return_if_fail (to_go != NULL);
-  g_return_if_fail (GTR_IS_MSG (to_go->data));
-
-  po = tab->priv->po;
-
-  current_msg = gtr_po_get_current_message (po);
-
-  /*
-   * Emitting message-edition-finished signal
-   */
-  if (!searching && !first_msg)
-    g_signal_emit (G_OBJECT (tab), signals[MESSAGE_EDITION_FINISHED],
-                   0, GTR_MSG (current_msg->data));
+  g_return_if_fail (GTR_IS_MSG (to_go));
 
   if (!tab->priv->blocking || first_msg)
     {
@@ -1285,14 +1241,14 @@ gtr_tab_message_go_to (GtrTab * tab,
             {
               gtk_notebook_set_current_page (GTK_NOTEBOOK
                                              (tab->priv->trans_notebook), 0);
-              gtr_tab_show_message (tab, to_go->data);
+              gtr_tab_show_message (tab, to_go);
             }
           else if (current_page == 0 && move == GTR_TAB_MOVE_PREV)
             {
               gtk_notebook_set_current_page (GTK_NOTEBOOK
                                              (tab->priv->trans_notebook),
                                              n_pages - 1);
-              gtr_tab_show_message (tab, to_go->data);
+              gtr_tab_show_message (tab, to_go);
             }
           else
             {
@@ -1308,7 +1264,7 @@ gtr_tab_message_go_to (GtrTab * tab,
             }
         }
       else
-        gtr_tab_show_message (tab, to_go->data);
+        gtr_tab_show_message (tab, to_go);
       first_msg = FALSE;
     }
   else
@@ -1319,7 +1275,7 @@ gtr_tab_message_go_to (GtrTab * tab,
    */
   if (!searching)
     g_signal_emit (G_OBJECT (tab), signals[SHOWED_MESSAGE], 0,
-                   GTR_MSG (to_go->data));
+                   GTR_MSG (to_go));
 }
 
 /**
@@ -1661,6 +1617,20 @@ gtr_tab_unblock_movement (GtrTab * tab)
   tab->priv->blocking = FALSE;
 }
 
+static gboolean
+_gtr_tab_finish_edition (GtrTab * tab)
+{
+  GList *current_msg;
+
+  current_msg = gtr_po_get_current_message (tab->priv->po);
+
+  /* movement is blocked/unblocked within the handler */
+  g_signal_emit (G_OBJECT (tab), signals[MESSAGE_EDITION_FINISHED],
+		 0, GTR_MSG (current_msg->data));
+
+  return !tab->priv->blocking;
+}
+
 /**
  * gtr_tab_go_to_next:
  * @tab: a #GtrTab
@@ -1670,14 +1640,16 @@ gtr_tab_unblock_movement (GtrTab * tab)
 void
 gtr_tab_go_to_next (GtrTab * tab)
 {
-  GtrPo *po;
+  GtrMsg *msg;
 
-  po = gtr_tab_get_po (tab);
-
-  gtr_tab_message_go_to (tab,
-                         g_list_next
-                         (gtr_po_get_current_message (po)),
-                         FALSE, GTR_TAB_MOVE_NEXT);
+  if (_gtr_tab_finish_edition (tab))
+    {
+      msg = gtr_message_table_navigate (GTR_MESSAGE_TABLE (tab->priv->message_table),
+                                        GTR_NAVIGATE_NEXT, NULL);
+      if (msg)
+        gtr_tab_message_go_to (tab, msg,
+                               FALSE, GTR_TAB_MOVE_NEXT);
+    }
 }
 
 /**
@@ -1689,14 +1661,16 @@ gtr_tab_go_to_next (GtrTab * tab)
 void
 gtr_tab_go_to_prev (GtrTab * tab)
 {
-  GtrPo *po;
+  GtrMsg *msg;
 
-  po = gtr_tab_get_po (tab);
-
-  gtr_tab_message_go_to (tab,
-                         g_list_previous
-                         (gtr_po_get_current_message (po)),
-                         FALSE, GTR_TAB_MOVE_PREV);
+  if (_gtr_tab_finish_edition (tab))
+    {
+      msg = gtr_message_table_navigate (GTR_MESSAGE_TABLE (tab->priv->message_table),
+                                        GTR_NAVIGATE_PREV, NULL);
+      if (msg)
+        gtr_tab_message_go_to (tab, msg,
+                               FALSE, GTR_TAB_MOVE_PREV);
+    }
 }
 
 /**
@@ -1708,14 +1682,16 @@ gtr_tab_go_to_prev (GtrTab * tab)
 void
 gtr_tab_go_to_first (GtrTab * tab)
 {
-  GtrPo *po;
+  GtrMsg *msg;
 
-  po = gtr_tab_get_po (tab);
-
-  gtr_tab_message_go_to (tab,
-                         g_list_first
-                         (gtr_po_get_current_message (po)),
-                         FALSE, GTR_TAB_MOVE_NONE);
+  if (_gtr_tab_finish_edition (tab))
+    {
+      msg = gtr_message_table_navigate (GTR_MESSAGE_TABLE (tab->priv->message_table),
+                                        GTR_NAVIGATE_FIRST, NULL);
+      if (msg)
+        gtr_tab_message_go_to (tab, msg,
+                               FALSE, GTR_TAB_MOVE_NONE);
+    }
 }
 
 /**
@@ -1727,14 +1703,16 @@ gtr_tab_go_to_first (GtrTab * tab)
 void
 gtr_tab_go_to_last (GtrTab * tab)
 {
-  GtrPo *po;
+  GtrMsg *msg;
 
-  po = gtr_tab_get_po (tab);
-
-  gtr_tab_message_go_to (tab,
-                         g_list_last
-                         (gtr_po_get_current_message (po)),
-                         FALSE, GTR_TAB_MOVE_NONE);
+  if (_gtr_tab_finish_edition (tab))
+    {
+      msg = gtr_message_table_navigate (GTR_MESSAGE_TABLE (tab->priv->message_table),
+                                        GTR_NAVIGATE_LAST, NULL);
+      if (msg)
+        gtr_tab_message_go_to (tab, msg,
+                               FALSE, GTR_TAB_MOVE_NONE);
+    }
 }
 
 /**
@@ -1748,12 +1726,14 @@ gtr_tab_go_to_last (GtrTab * tab)
 gboolean
 gtr_tab_go_to_next_fuzzy (GtrTab * tab)
 {
-  GtrPo *po;
-  GList *msg;
+  GtrMsg *msg;
 
-  po = gtr_tab_get_po (tab);
+  if (!_gtr_tab_finish_edition (tab))
+    return FALSE;
 
-  msg = gtr_po_get_next_fuzzy (po);
+  msg = gtr_message_table_navigate (GTR_MESSAGE_TABLE (tab->priv->message_table),
+                                    GTR_NAVIGATE_NEXT,
+                                    gtr_msg_is_fuzzy);
   if (msg != NULL)
     {
       gtr_tab_message_go_to (tab, msg, FALSE, GTR_TAB_MOVE_NONE);
@@ -1774,12 +1754,14 @@ gtr_tab_go_to_next_fuzzy (GtrTab * tab)
 gboolean
 gtr_tab_go_to_prev_fuzzy (GtrTab * tab)
 {
-  GtrPo *po;
-  GList *msg;
+  GtrMsg *msg;
 
-  po = gtr_tab_get_po (tab);
+  if (!_gtr_tab_finish_edition (tab))
+    return FALSE;
 
-  msg = gtr_po_get_prev_fuzzy (po);
+  msg = gtr_message_table_navigate (GTR_MESSAGE_TABLE (tab->priv->message_table),
+                                    GTR_NAVIGATE_PREV,
+                                    gtr_msg_is_fuzzy);
   if (msg != NULL)
     {
       gtr_tab_message_go_to (tab, msg, FALSE, GTR_TAB_MOVE_NONE);
@@ -1789,6 +1771,18 @@ gtr_tab_go_to_prev_fuzzy (GtrTab * tab)
   return FALSE;
 }
 
+static gboolean
+message_is_untranslated (GtrMsg * msg)
+{
+  return !gtr_msg_is_translated (msg);
+}
+
+static gboolean
+message_is_fuzzy_or_untranslated (GtrMsg * msg)
+{
+  return gtr_msg_is_fuzzy (msg) || !gtr_msg_is_translated (msg);
+}
+
 /**
  * gtr_tab_go_to_next_untrans:
  * @tab: a #GtrTab
@@ -1800,12 +1794,14 @@ gtr_tab_go_to_prev_fuzzy (GtrTab * tab)
 gboolean
 gtr_tab_go_to_next_untrans (GtrTab * tab)
 {
-  GtrPo *po;
-  GList *msg;
+  GtrMsg *msg;
 
-  po = gtr_tab_get_po (tab);
+  if (!_gtr_tab_finish_edition (tab))
+    return FALSE;
 
-  msg = gtr_po_get_next_untrans (po);
+  msg = gtr_message_table_navigate (GTR_MESSAGE_TABLE (tab->priv->message_table),
+                                    GTR_NAVIGATE_NEXT,
+                                    message_is_untranslated);
   if (msg != NULL)
     {
       gtr_tab_message_go_to (tab, msg, FALSE, GTR_TAB_MOVE_NONE);
@@ -1826,12 +1822,14 @@ gtr_tab_go_to_next_untrans (GtrTab * tab)
 gboolean
 gtr_tab_go_to_prev_untrans (GtrTab * tab)
 {
-  GtrPo *po;
-  GList *msg;
+  GtrMsg *msg;
 
-  po = gtr_tab_get_po (tab);
+  if (!_gtr_tab_finish_edition (tab))
+    return FALSE;
 
-  msg = gtr_po_get_prev_untrans (po);
+  msg = gtr_message_table_navigate (GTR_MESSAGE_TABLE (tab->priv->message_table),
+                                    GTR_NAVIGATE_PREV,
+                                    message_is_untranslated);
   if (msg != NULL)
     {
       gtr_tab_message_go_to (tab, msg, FALSE, GTR_TAB_MOVE_NONE);
@@ -1852,12 +1850,14 @@ gtr_tab_go_to_prev_untrans (GtrTab * tab)
 gboolean
 gtr_tab_go_to_next_fuzzy_or_untrans (GtrTab * tab)
 {
-  GtrPo *po;
-  GList *msg;
+  GtrMsg *msg;
 
-  po = gtr_tab_get_po (tab);
+  if (_gtr_tab_finish_edition (tab))
+    return FALSE;
 
-  msg = gtr_po_get_next_fuzzy_or_untrans (po);
+  msg = gtr_message_table_navigate (GTR_MESSAGE_TABLE (tab->priv->message_table),
+                                    GTR_NAVIGATE_PREV,
+                                    message_is_fuzzy_or_untranslated);
   if (msg != NULL)
     {
       gtr_tab_message_go_to (tab, msg, FALSE, GTR_TAB_MOVE_NONE);
@@ -1878,12 +1878,14 @@ gtr_tab_go_to_next_fuzzy_or_untrans (GtrTab * tab)
 gboolean
 gtr_tab_go_to_prev_fuzzy_or_untrans (GtrTab * tab)
 {
-  GtrPo *po;
-  GList *msg;
+  GtrMsg *msg;
 
-  po = gtr_tab_get_po (tab);
+  if (!_gtr_tab_finish_edition (tab))
+    return FALSE;
 
-  msg = gtr_po_get_prev_fuzzy_or_untrans (po);
+  msg = gtr_message_table_navigate (GTR_MESSAGE_TABLE (tab->priv->message_table),
+                                    GTR_NAVIGATE_PREV,
+                                    message_is_fuzzy_or_untranslated);
   if (msg != NULL)
     {
       gtr_tab_message_go_to (tab, msg, FALSE, GTR_TAB_MOVE_NONE);
@@ -1907,12 +1909,14 @@ gtr_tab_go_to_number (GtrTab * tab, gint number)
   GtrPo *po;
   GList *msg;
 
-  po = gtr_tab_get_po (tab);
+  if (!_gtr_tab_finish_edition (tab))
+    return;
 
+  po = gtr_tab_get_po (tab);
   msg = gtr_po_get_msg_from_number (po, number);
   if (msg != NULL)
     {
-      gtr_tab_message_go_to (tab, msg, FALSE, GTR_TAB_MOVE_NONE);
+      gtr_tab_message_go_to (tab, msg->data, FALSE, GTR_TAB_MOVE_NONE);
     }
 }
 
