@@ -41,124 +41,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-static const gchar *badwords[] = {
-  "a",
-  //"all",
-  "an",
-  //"are",
-  //"can",
-  //"for",
-  //"from",
-  "have",
-  //"it",
-  //"may",
-  //"not",
-  "of",
-  //"that",
-  "the",
-  //"this",
-  //"was",
-  "will",
-  //"with",
-  //"you",
-  //"your",
-  NULL
-};
-
-static gboolean
-check_good_word (const gchar * word, gchar ** badwords)
-{
-  gboolean check = TRUE;
-  gchar *lower = g_utf8_strdown (word, -1);
-  gint i = 0;
-
-  while (badwords[i] != NULL)
-    {
-      gchar *lower_collate = g_utf8_collate_key (lower, -1);
-
-      if (strcmp (lower_collate, badwords[i]) == 0)
-        {
-          check = FALSE;
-          g_free (lower_collate);
-          break;
-        }
-      i++;
-      g_free (lower_collate);
-    }
-  return check;
-}
-
-/**
- * gtr_utils_split_string_in_words:
- * @string: the text to process
- *
- * Process a text and split it in words using pango.
- * 
- * Returns: an array of words of the processed text
- */
-gchar **
-gtr_utils_split_string_in_words (const gchar * string)
-{
-  PangoLanguage *lang = pango_language_from_string ("en");
-  PangoLogAttr *attrs;
-  GPtrArray *array;
-  gint char_len;
-  gint i = 0;
-  gchar *s;
-  gchar *start = NULL;
-  static gchar **badwords_collate = NULL;
-
-  if (badwords_collate == NULL)
-    {
-      gint words_size = g_strv_length ((gchar **) badwords);
-      gint x = 0;
-
-      badwords_collate = g_new0 (gchar *, words_size + 1);
-
-      while (badwords[x] != NULL)
-        {
-          badwords_collate[x] = g_utf8_collate_key (badwords[x], -1);
-          x++;
-        }
-      badwords_collate[x] = NULL;
-    }
-
-  char_len = g_utf8_strlen (string, -1);
-  attrs = g_new (PangoLogAttr, char_len + 1);
-
-  pango_get_log_attrs (string,
-                       strlen (string), -1, lang, attrs, char_len + 1);
-
-  array = g_ptr_array_new ();
-
-  s = (gchar *) string;
-  while (i <= char_len)
-    {
-      gchar *end;
-
-      if (attrs[i].is_word_start)
-        start = s;
-      if (attrs[i].is_word_end)
-        {
-          gchar *word;
-
-          end = s;
-          word = g_strndup (start, end - start);
-
-          if (check_good_word (word, badwords_collate))
-            g_ptr_array_add (array, word);
-        }
-
-      i++;
-      s = g_utf8_next_char (s);
-    }
-
-  g_free (attrs);
-  g_ptr_array_add (array, NULL);
-
-  return (gchar **) g_ptr_array_free (array, FALSE);
-}
-
 xmlDocPtr
 gtr_xml_new_doc (const gchar * name)
 {
@@ -699,69 +581,6 @@ gtr_utils_get_current_year (void)
   return year;
 }
 
-/**
- * gtr_utils_scan_dir:
- * @dir: the dir to parse
- * @list: the list where to store the GFiles
- * @po_name: the name of the specific po file to search or NULL.
- *
- * Scans the directory and subdirectories of @dir looking for filenames remained
- * with .po or files that matches @po_name. @list must be freed with
- * g_slist_free_full (list, g_object_unref).
- */
-void
-gtr_utils_scan_dir (GFile * dir, GSList ** list, const gchar * po_name)
-{
-  GFileInfo *info;
-  GError *error;
-  GFile *file;
-  GFileEnumerator *enumerator;
-
-  error = NULL;
-  enumerator = g_file_enumerate_children (dir,
-                                          G_FILE_ATTRIBUTE_STANDARD_NAME,
-                                          G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-                                          NULL, &error);
-  if (enumerator)
-    {
-      error = NULL;
-
-      while ((info =
-              g_file_enumerator_next_file (enumerator, NULL, &error)) != NULL)
-        {
-          const gchar *name;
-          gchar *filename;
-
-          name = g_file_info_get_name (info);
-          file = g_file_get_child (dir, name);
-
-          if (po_name != NULL)
-            {
-              if (g_str_has_suffix (po_name, ".po"))
-                filename = g_strdup (po_name);
-              else
-                filename = g_strconcat (po_name, ".po", NULL);
-            }
-          else
-            filename = g_strdup (".po");
-
-          if (g_str_has_suffix (name, filename))
-            *list = g_slist_prepend (*list, file);
-          g_free (filename);
-
-          gtr_utils_scan_dir (file, list, po_name);
-          g_object_unref (info);
-        }
-      g_file_enumerator_close (enumerator, NULL, NULL);
-      g_object_unref (enumerator);
-
-      if (error)
-        {
-          g_warning ("%s", error->message);
-        }
-    }
-}
-
 gchar *
 gtr_utils_reduce_path (const gchar * path)
 {
