@@ -70,7 +70,7 @@ static void gtr_window_cmd_edit_toolbar (GtkAction * action,
                                          GtrWindow * window);
 
 
-G_DEFINE_TYPE (GtrWindow, gtr_window, GTK_TYPE_WINDOW)
+G_DEFINE_TYPE (GtrWindow, gtr_window, GTK_TYPE_APPLICATION_WINDOW)
 
 struct _GtrWindowPrivate
 {
@@ -120,7 +120,6 @@ static const GtkActionEntry always_sensitive_entries[] = {
   {"Go", NULL, N_("_Go")},
   {"Tools", NULL, N_("_Tools")},
   {"Documents", NULL, N_("_Documents")},
-  {"Help", NULL, N_("_Help")},
 
   /* File menu */
   {"FileOpen", GTK_STOCK_OPEN, NULL, "<control>O",
@@ -135,17 +134,8 @@ static const GtkActionEntry always_sensitive_entries[] = {
   /* Edit menu */
   {"EditToolbar", NULL, N_("T_oolbar"), NULL, NULL,
    G_CALLBACK (gtr_window_cmd_edit_toolbar)},
-  {"EditPreferences", GTK_STOCK_PREFERENCES, NULL, NULL,
-   N_("Edit gtranslator preferences"),
-   G_CALLBACK (gtr_actions_edit_preferences)},
   {"EditHeader", GTK_STOCK_PROPERTIES, N_("_Header..."), NULL, NULL,
    G_CALLBACK (gtr_actions_edit_header)},
-
-  /* Help menu */
-  {"HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", NULL,
-   G_CALLBACK (gtr_cmd_help_contents)},
-  {"HelpAbout", GTK_STOCK_ABOUT, NULL, NULL, NULL,
-   G_CALLBACK (gtr_about_dialog)},
 };
 
 /* Normal items */
@@ -900,6 +890,12 @@ showed_message_cb (GtrTab * tab, GtrMsg * msg, GtrWindow * window)
   _gtr_window_set_sensitive_according_to_message (window, gtr_tab_get_po (tab));
 }
 
+static void
+selection_changed_cb (GtrTab * tab, GtrWindow * window)
+{
+  set_sensitive_according_to_tab (window, tab);
+}
+
 static void
 notebook_tab_added (GtkNotebook * notebook,
                     GtkWidget * child, guint page_num, GtrWindow * window)
@@ -948,6 +944,8 @@ notebook_tab_added (GtkNotebook * notebook,
                           window);
   g_signal_connect_after (child, "showed_message",
                           G_CALLBACK (showed_message_cb), window);
+  g_signal_connect (child, "selection-changed",
+                    G_CALLBACK (selection_changed_cb), window);
 
   g_signal_connect (gtr_tab_get_po (tab),
                     "notify::state", G_CALLBACK (sync_state), window);
@@ -1369,12 +1367,54 @@ on_profile_modified (GtrProfileManager *manager,
 }
 
 static void
-gtr_window_draw (GtrWindow * window)
+extension_added (PeasExtensionSet *extensions,
+                 PeasPluginInfo   *info,
+                 PeasExtension    *exten,
+                 GtrWindow        *window)
 {
+  gtr_window_activatable_activate (GTR_WINDOW_ACTIVATABLE (exten));
+}
+
+static void
+extension_removed (PeasExtensionSet *extensions,
+                   PeasPluginInfo   *info,
+                   PeasExtension    *exten,
+                   GtrWindow        *window)
+{
+  gtr_window_activatable_deactivate (GTR_WINDOW_ACTIVATABLE (exten));
+
+  /* Ensure update of ui manager, because we suspect it does something
+   * with expected static strings in the type module (when unloaded the
+   * strings don't exist anymore, and ui manager updates in an idle
+   * func) */
+  gtk_ui_manager_ensure_update (window->priv->ui_manager);
+}
+
+static void
+gtr_window_init (GtrWindow * window)
+{
+  GtkTargetList *tl;
   GtkWidget *widget;
   GError *error = NULL;
   GtkActionGroup *action_group;
-  GtrWindowPrivate *priv = window->priv;
+  GtrWindowPrivate *priv;
+
+  window->priv = GTR_WINDOW_GET_PRIVATE (window);
+  priv = window->priv;
+
+  priv->state_settings = g_settings_new ("org.gnome.gtranslator.state.window");
+
+  /* Profile manager */
+  priv->prof_manager = gtr_profile_manager_get_default ();
+
+  g_signal_connect (priv->prof_manager, "active-profile-changed",
+                    G_CALLBACK (on_active_profile_changed), window);
+  g_signal_connect (priv->prof_manager, "profile-added",
+                    G_CALLBACK (on_profile_added), window);
+  g_signal_connect (priv->prof_manager, "profile-removed",
+                    G_CALLBACK (on_profile_removed), window);
+  g_signal_connect (priv->prof_manager, "profile-modified",
+                    G_CALLBACK (on_profile_modified), window);
 
   /* Main box */
   priv->main_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
@@ -1483,57 +1523,7 @@ gtr_window_draw (GtrWindow * window)
 
   /* statusbar & progress bar */
   create_statusbar (window);
-}
 
-static void
-extension_added (PeasExtensionSet *extensions,
-                 PeasPluginInfo   *info,
-                 PeasExtension    *exten,
-                 GtrWindow        *window)
-{
-  gtr_window_activatable_activate (GTR_WINDOW_ACTIVATABLE (exten));
-}
-
-static void
-extension_removed (PeasExtensionSet *extensions,
-                   PeasPluginInfo   *info,
-                   PeasExtension    *exten,
-                   GtrWindow        *window)
-{
-  gtr_window_activatable_deactivate (GTR_WINDOW_ACTIVATABLE (exten));
-
-  /* Ensure update of ui manager, because we suspect it does something
-   * with expected static strings in the type module (when unloaded the
-   * strings don't exist anymore, and ui manager updates in an idle
-   * func) */
-  gtk_ui_manager_ensure_update (window->priv->ui_manager);
-}
-
-static void
-gtr_window_init (GtrWindow * window)
-{
-  GtkTargetList *tl;
-
-  window->priv = GTR_WINDOW_GET_PRIVATE (window);
-
-  window->priv->state_settings = g_settings_new ("org.gnome.gtranslator.state.window");
-
-  window->priv->dispose_has_run = FALSE;
-
-  /* Profile manager */
-  window->priv->prof_manager = gtr_profile_manager_get_default ();
-
-  g_signal_connect (window->priv->prof_manager, "active-profile-changed",
-                    G_CALLBACK (on_active_profile_changed), window);
-  g_signal_connect (window->priv->prof_manager, "profile-added",
-                    G_CALLBACK (on_profile_added), window);
-  g_signal_connect (window->priv->prof_manager, "profile-removed",
-                    G_CALLBACK (on_profile_removed), window);
-  g_signal_connect (window->priv->prof_manager, "profile-modified",
-                    G_CALLBACK (on_profile_modified), window);
-
-  /* set up the window widgets */
-  gtr_window_draw (window);
 
   set_sensitive_according_to_window (window);
 
@@ -1615,28 +1605,10 @@ gtr_window_dispose (GObject * object)
       priv->dispose_has_run = TRUE;
     }
 
-  if (priv->state_settings)
-    {
-      g_object_unref (priv->state_settings);
-      priv->state_settings = NULL;
-    }
-
-  if (priv->ui_manager)
-    {
-      g_object_unref (priv->ui_manager);
-      priv->ui_manager = NULL;
-    }
-  if (priv->action_group)
-    {
-      g_object_unref (priv->action_group);
-      priv->action_group = NULL;
-    }
-
-  if (priv->prof_manager != NULL)
-    {
-      g_object_unref (priv->prof_manager);
-      priv->prof_manager = NULL;
-    }
+  g_clear_object (&priv->state_settings);
+  g_clear_object (&priv->ui_manager);
+  g_clear_object (&priv->action_group);
+  g_clear_object (&priv->prof_manager);
 
   /* Now that there have broken some reference loops,
    * force collection again.
